const mongoose = require('mongoose');
const config = require('../config/database');

const AreaSchema = mongoose.Schema({
    city: { type: String },
    state: { type: String },
    district: { type: String },
    country: { type: String },
    createdBy: { type: String },
    type: { type: String, default: "city" },
    createdAt: { type: Date, default: Date.now() }
});

const Area = (module.exports = mongoose.model('area', AreaSchema));

module.exports.addArea = (newArea, callback) => {
    newArea.save(callback);
};

// ****-- - Get Data by Name----****
module.exports.getCountryByName = (country, callback) => {
    Area.findOne({ country: country }, callback);
};

module.exports.getStateByName = (state, callback) => {
    Area.findOne({ state: state }, callback);
};


module.exports.getDistrictByName = (dist, callback) => {
    Area.findOne({ district: dist }, callback);
};

module.exports.getCityByName = (city, callback) => {
    Area.findOne({ city: city }, callback);
};
// ****------****

module.exports.getCitiesByCountry = (country, callback) => {
    Area.find({ $and: [{ type: "city" }, { country: country }] }, callback);
};


// get all states
module.exports.getAllStates = (callback) => {
    // Area.find({ $or: [{ state: { $exists: true } }, { type: "state" }] }, callback);
    Area.aggregate([
        {
            $group: {
                _id: "$state",
                count: {
                    $sum: 1
                }
            }
        },
        {
            $sort: { "_id": 1 }
        }
    ], callback);
};

// ** Update Many Functions Start ** //

// update country name
module.exports.updateCountry = (country, newCountry, callback) => {
    Area.updateMany({ country: country }, { $set: { country: newCountry } }, callback);
}

// update state name
module.exports.updateState = (state, newState, callback) => {
    Area.updateMany({ state: state }, { $set: { state: newState } }, callback);
};

// update district name
module.exports.updateDistrict = (district, newDistrict, callback) => {
    Area.updateMany({ district: district }, { $set: { district: newDistrict } }, callback);
};

// update city name
module.exports.updateCity = (city, newCity, callback) => {
    Area.updateMany({ city: city }, { $set: { city: newCity } }, callback);
};

// ** Update Many Functions End ** //

// get states by country
module.exports.getStatesByCountry = (country, callback) => {
    // Area.find({ $and: [{ country: country }, { state: { $exists: true } }] }, callback);
    Area.aggregate([
        {
            $match: { country: country }
        },
        {
            $group: {
                _id: "$state",
                count: {
                    $sum: 1
                }
            }
        },
        {
            $sort: { "_id": 1 }
        }
    ], callback);
};

// get districts by state
module.exports.getDistrictByState = (state, callback) => {
    // Area.find({ $and: [{ state: state }, { district: { $exists: true } }] }, callback);
    Area.aggregate([
        {
            $match: { state: state }
        },
        {
            $group: {
                _id: "$district",
                count: {
                    $sum: 1
                }
            }
        },
        {
            $sort: { "_id": 1 }
        }
    ], callback);
};

// get cities by district
module.exports.getCitiesByDistrict = (district, callback) => {
    // Area.find({ $and: [{ district: dist }, { city: { $exists: true } }] }, callback);
    Area.aggregate([
        {
            $match: { district: district }
        },
        {
            $group: {
                _id: "$city",
                district: { $push: "$district" },
                country: { $push: "$country" },
                state: { $push: "$state" },
                count: {
                    $sum: 1
                }
            }
        },
        {
            $sort: { "_id": 1 }
        }
    ], callback);
};


// get cities by state
module.exports.getCitiesByState = (state, callback) => {
    // Area.find({ $and: [{ district: dist }, { city: { $exists: true } }] }, callback);
    Area.aggregate([
        {
            $match: { state: state }
        },
        {
            $group: {
                _id: "$city",
                district: { $push: "$district" },
                country: { $push: "$country" },
                state: { $push: "$state" },
                count: {
                    $sum: 1
                }
            }
        },
        {
            $sort: { "_id": 1 }
        }
    ], callback);
};

// get data by state
module.exports.getDataByState = (stateName, callback) => {
    Area.find({ state: stateName }, callback);
}

// getCities
module.exports.getCities = callback => {
    Area.find({ type: 'city' }, callback);
};

// get City By Id
module.exports.getCityById = (id, callback) => {
    Area.findOne({ _id: id }, callback);
};

// get districts
module.exports.getAllDistricts = callback => {
    // Area.find({ type: 'district' }, callback);
    Area.aggregate([{
        $group: {
            _id: "$district",
            count: {
                $sum: 1
            }
        }
    }], callback);
};

// get countries
module.exports.getCountries = callback => {
    // Area.find({ country: { $exists: true } }, callback);
    Area.aggregate([{
        $group: {
            _id: "$country",
            count: {
                $sum: 1
            }
        }
    }], callback);
};

// **** Get multiple States/countries/cities/districts by name 
module.exports.getCountriesByName = (country, callback) => {
    const q = new RegExp(country, 'gi');
    Area.find({ country: q }, callback);
}

module.exports.getStatesByName = (state, callback) => {
    const q = new RegExp(state, 'gi');
    Area.find({ state: q }, callback);
}

module.exports.getDistrictsByName = (dist, callback) => {
    const q = new RegExp(dist, 'gi');
    Area.find({ district: q }, callback);
}


module.exports.getCitiesByName = (city, callback) => {
    const q = new RegExp(city, 'gi');
    Area.find({ city: q }, callback);
}

// ******* Delete States/Cities/Countries/Districts *******//
module.exports.deleteCountry = (country, callback) => {
    Area.findOneAndDelete({ country: country }, callback);
}
module.exports.deleteState = (state, callback) => {
    Area.findOneAndDelete({ state: state }, callback);
}
module.exports.deleteDistrict = (district, callback) => {
    Area.findOneAndDelete({ district: district }, callback);
}
module.exports.deleteCity = (city, callback) => {
    Area.findOneAndDelete({ city: city }, callback);
}

// check for duplicate
module.exports.duplicateCheck = (city, dist, st, callback) => {
    Area.findOne({ city: city, district: dist, state: st }, callback);
}


// testing inserting array using insert command
module.exports.insertArrayofAreas = (areas, callback) => {
    Area.insertMany(areas, callback);
};